@@ -1,5 +1,5 @@
 import logging
-import pkg_resources
+import importlib.metadata
 
 import pint
 
@@ -9,14 +9,12 @@ logging.basicConfig(level=logging.WARNING)
 logging.captureWarnings(True)
 pint.util.logger.setLevel(logging.ERROR)  # Mute Pint warnings
 
-# Single-sourcing the package version
-version_file = pkg_resources.resource_filename("cript", "VERSION.txt")
-with open(version_file, "r") as fr:
-    __version__ = fr.read().strip()
 
-VERSION = __version__
+# Single-sourcing the package version
+__version__ = importlib.metadata.version("cript")
 __short_version__ = __version__.rpartition(".")[0]
 
+__api_version__ = "0.5.0"
 
 # Instantiate the Pint unit registry
 # https://pint.readthedocs.io/en/stable/tutorial.html#using-pint-in-your-projects
@@ -25,6 +23,7 @@ pint_ureg = pint.UnitRegistry(autoconvert_offset_to_baseunit=True)
 
 
 from cript import exceptions  # noqa 401 402
+from cript.paginator import Paginator
 from cript.nodes import (  # noqa 402
     User,
     Group,
@@ -35,6 +34,8 @@ from cript.nodes import (  # noqa 402
     File,
     Data,
     Condition,
+    SoftwareConfiguration,
+    Computation,
     Property,
     Identifier,
     Material,
@@ -44,6 +45,11 @@ from cript.nodes import (  # noqa 402
     Equipment,
     Process,
     Experiment,
+    Software,
+    Parameter,
+    Algorithm,
+    ComputationalProcess,
+    ComputationalForcefield,
 )
 
 NODE_CLASSES = [
@@ -56,6 +62,8 @@ NODE_CLASSES = [
     File,
     Data,
     Condition,
+    SoftwareConfiguration,
+    Computation,
     Property,
     Identifier,
     Material,
@@ -65,6 +73,14 @@ NODE_CLASSES = [
     Equipment,
     Process,
     Experiment,
+    Software,
+    Parameter,
+    Algorithm,
+    ComputationalProcess,
+    ComputationalForcefield,
 ]
 
+NODE_NAMES: list[str] = [node.node_name.lower() for node in NODE_CLASSES]
+
 from cript.api import API  # noqa 401 402
+from cript.api_local import APILocal
