@@ -2,6 +2,7 @@
 from cript.nodes.primary.user import User
 from cript.nodes.primary.group import Group
 from cript.nodes.primary.reference import Reference
+from cript.nodes.primary.software import Software
 from cript.nodes.secondary.citation import Citation
 from cript.nodes.primary.project import Project
 from cript.nodes.primary.collection import Collection
@@ -9,6 +10,8 @@ from cript.nodes.primary.experiment import Experiment
 from cript.nodes.primary.data import Data
 from cript.nodes.primary.file import File
 from cript.nodes.secondary.condition import Condition
+from cript.nodes.secondary.software_configuration import SoftwareConfiguration
+from cript.nodes.primary.computation import Computation
 from cript.nodes.secondary.property import Property
 from cript.nodes.secondary.identifier import Identifier
 from cript.nodes.secondary.quantity import Quantity
@@ -17,3 +20,7 @@ from cript.nodes.primary.inventory import Inventory
 from cript.nodes.secondary.ingredient import Ingredient
 from cript.nodes.secondary.equipment import Equipment
 from cript.nodes.primary.process import Process
+from cript.nodes.secondary.parameter import Parameter
+from cript.nodes.secondary.algorithm import Algorithm
+from cript.nodes.primary.computational_process import ComputationalProcess
+from cript.nodes.secondary.computational_forcefield import ComputationalForcefield
