@@ -35,6 +35,11 @@ def convert_to_api_url(url: str):
     scheme = parsed_url.scheme
     netloc = parsed_url.netloc
     path = parsed_url.path
+
+    # Return original URL if in correct format
+    if path.startswith("/api/"):
+        return url
+
     return f"{scheme}://{netloc}/api{path}"
 
 
@@ -45,7 +50,7 @@ def auto_assign_group(group, parent):
 
     :param group: Current value of the node's group field.
     :param parent: The parent node of the relevant node.
-    :return: The :class:`Group` object that will be assigned to the node.
+    :return: The `Group` object that will be assigned to the node.
     :rtype: cript.nodes.Group
     """
     if parent and not group:
